# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/about.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AboutDialog(object):
    def setupUi(self, AboutDialog):
        AboutDialog.setObjectName("AboutDialog")
        AboutDialog.resize(506, 595)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(AboutDialog)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.logoLabel = QtWidgets.QLabel(AboutDialog)
        self.logoLabel.setMaximumSize(QtCore.QSize(16777215, 16777215))
        self.logoLabel.setText("")
        self.logoLabel.setPixmap(QtGui.QPixmap(":/images/logo"))
        self.logoLabel.setScaledContents(False)
        self.logoLabel.setObjectName("logoLabel")
        self.verticalLayout_2.addWidget(self.logoLabel, 0, QtCore.Qt.AlignHCenter)
        self.descriptionLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setItalic(True)
        self.descriptionLabel.setFont(font)
        self.descriptionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.descriptionLabel.setObjectName("descriptionLabel")
        self.verticalLayout_2.addWidget(self.descriptionLabel)
        self.versionLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.versionLabel.setFont(font)
        self.versionLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.versionLabel.setObjectName("versionLabel")
        self.verticalLayout_2.addWidget(self.versionLabel)
        self.authorTitleLabel = QtWidgets.QLabel(AboutDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.authorTitleLabel.setFont(font)
        self.authorTitleLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.authorTitleLabel.setObjectName("authorTitleLabel")
        self.verticalLayout_2.addWidget(self.authorTitleLabel)
        self.authorListLabel = QtWidgets.QLabel(AboutDialog)
        self.authorListLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.authorListLabel.setObjectName("authorListLabel")
        self.verticalLayout_2.addWidget(self.authorListLabel)

        self.retranslateUi(AboutDialog)
        QtCore.QMetaObject.connectSlotsByName(AboutDialog)

    def retranslateUi(self, AboutDialog):
        #  _translate = QtCore.QCoreApplication.translate
        AboutDialog.setWindowTitle(_("About"))
        self.descriptionLabel.setText(_("A free software desktop client for Duniter. \n"
"Manage your Ğ1 accounts securely and easily!"))
        self.versionLabel.setText(_("Version x.y.z"))
        self.authorTitleLabel.setText(_("Author"))
        self.authorListLabel.setText(_("Author list"))
import tikka.slots.pyqt.resources.icons.index_rc
