# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountWidget(object):
    def setupUi(self, AccountWidget):
        AccountWidget.setObjectName("AccountWidget")
        AccountWidget.resize(615, 384)
        AccountWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(AccountWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.safeImageLabel = QtWidgets.QLabel(AccountWidget)
        self.safeImageLabel.setMaximumSize(QtCore.QSize(100, 100))
        self.safeImageLabel.setBaseSize(QtCore.QSize(200, 0))
        self.safeImageLabel.setText("")
        self.safeImageLabel.setPixmap(QtGui.QPixmap(":/images/safe"))
        self.safeImageLabel.setScaledContents(True)
        self.safeImageLabel.setObjectName("safeImageLabel")
        self.horizontalLayout.addWidget(self.safeImageLabel)
        self.balanceLabel = QtWidgets.QLabel(AccountWidget)
        font = QtGui.QFont()
        font.setPointSize(40)
        self.balanceLabel.setFont(font)
        self.balanceLabel.setObjectName("balanceLabel")
        self.horizontalLayout.addWidget(self.balanceLabel)
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.verticalLayout = QtWidgets.QVBoxLayout()
        self.verticalLayout.setObjectName("verticalLayout")
        self.refreshButton = QtWidgets.QPushButton(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.refreshButton.sizePolicy().hasHeightForWidth())
        self.refreshButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshButton.setIcon(icon)
        self.refreshButton.setObjectName("refreshButton")
        self.verticalLayout.addWidget(self.refreshButton)
        self.transferButton = QtWidgets.QPushButton(AccountWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.transferButton.sizePolicy().hasHeightForWidth())
        self.transferButton.setSizePolicy(sizePolicy)
        icon1 = QtGui.QIcon()
        icon1.addPixmap(QtGui.QPixmap(":/icons/payment_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.transferButton.setIcon(icon1)
        self.transferButton.setObjectName("transferButton")
        self.verticalLayout.addWidget(self.transferButton)
        self.horizontalLayout.addLayout(self.verticalLayout)
        self.verticalLayout_2.addLayout(self.horizontalLayout)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setContentsMargins(6, -1, 0, -1)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.lockStatusIcon = QtWidgets.QLabel(AccountWidget)
        self.lockStatusIcon.setMaximumSize(QtCore.QSize(16, 16))
        self.lockStatusIcon.setText("")
        self.lockStatusIcon.setPixmap(QtGui.QPixmap(":/images/locked_cartoon_400x400.png"))
        self.lockStatusIcon.setScaledContents(True)
        self.lockStatusIcon.setObjectName("lockStatusIcon")
        self.horizontalLayout_2.addWidget(self.lockStatusIcon)
        self.addressLabel = QtWidgets.QLabel(AccountWidget)
        self.addressLabel.setText("")
        self.addressLabel.setObjectName("addressLabel")
        self.horizontalLayout_2.addWidget(self.addressLabel)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        spacerItem1 = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem1)

        self.retranslateUi(AccountWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountWidget)

    def retranslateUi(self, AccountWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountWidget.setWindowTitle(_("Account"))
        self.balanceLabel.setText(_("0"))
        self.refreshButton.setText(_("Refresh"))
        self.transferButton.setText(_("Transfer"))
import tikka.slots.pyqt.resources.icons.index_rc
