# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/account_list.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountListWidget(object):
    def setupUi(self, AccountListWidget):
        AccountListWidget.setObjectName("AccountListWidget")
        AccountListWidget.resize(565, 296)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountListWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.label = QtWidgets.QLabel(AccountListWidget)
        font = QtGui.QFont()
        font.setFamily("FreeSans")
        font.setBold(True)
        font.setWeight(75)
        self.label.setFont(font)
        self.label.setObjectName("label")
        self.verticalLayout.addWidget(self.label, 0, QtCore.Qt.AlignHCenter)
        self.tableWidget = QtWidgets.QTableWidget(AccountListWidget)
        self.tableWidget.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.tableWidget.setVerticalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.tableWidget.setSizeAdjustPolicy(QtWidgets.QAbstractScrollArea.AdjustToContents)
        self.tableWidget.setAutoScroll(True)
        self.tableWidget.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.tableWidget.setObjectName("tableWidget")
        self.tableWidget.setColumnCount(2)
        self.tableWidget.setRowCount(0)
        item = QtWidgets.QTableWidgetItem()
        self.tableWidget.setHorizontalHeaderItem(0, item)
        item = QtWidgets.QTableWidgetItem()
        item.setTextAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignVCenter)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        item.setFont(font)
        self.tableWidget.setHorizontalHeaderItem(1, item)
        self.tableWidget.horizontalHeader().setDefaultSectionSize(50)
        self.tableWidget.horizontalHeader().setMinimumSectionSize(50)
        self.tableWidget.horizontalHeader().setStretchLastSection(True)
        self.tableWidget.verticalHeader().setVisible(False)
        self.verticalLayout.addWidget(self.tableWidget)

        self.retranslateUi(AccountListWidget)
        QtCore.QMetaObject.connectSlotsByName(AccountListWidget)

    def retranslateUi(self, AccountListWidget):
        #  _translate = QtCore.QCoreApplication.translate
        AccountListWidget.setWindowTitle(_("Account list"))
        self.label.setText(_("Account list"))
        item = self.tableWidget.horizontalHeaderItem(1)
        item.setText(_("Address"))
import tikka.slots.pyqt.resources.icons.index_rc
