drop table IF exists currency;
drop table IF exists accounts;
drop table IF exists wallets;
drop table IF exists tabs;
drop table IF exists preferences;
drop table if exists entry_points;
drop table if exists nodes;
