# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/account_import.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_AccountImportDialog(object):
    def setupUi(self, AccountImportDialog):
        AccountImportDialog.setObjectName("AccountImportDialog")
        AccountImportDialog.resize(961, 400)
        self.verticalLayout = QtWidgets.QVBoxLayout(AccountImportDialog)
        self.verticalLayout.setObjectName("verticalLayout")
        self.imagelabel = QtWidgets.QLabel(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.imagelabel.sizePolicy().hasHeightForWidth())
        self.imagelabel.setSizePolicy(sizePolicy)
        self.imagelabel.setMinimumSize(QtCore.QSize(0, 0))
        self.imagelabel.setMaximumSize(QtCore.QSize(200, 200))
        self.imagelabel.setSizeIncrement(QtCore.QSize(0, 0))
        self.imagelabel.setBaseSize(QtCore.QSize(0, 0))
        self.imagelabel.setText("")
        self.imagelabel.setPixmap(QtGui.QPixmap(":/images/safe_background"))
        self.imagelabel.setScaledContents(True)
        self.imagelabel.setAlignment(QtCore.Qt.AlignCenter)
        self.imagelabel.setObjectName("imagelabel")
        self.verticalLayout.addWidget(self.imagelabel, 0, QtCore.Qt.AlignHCenter)
        self.formLayout = QtWidgets.QFormLayout()
        self.formLayout.setObjectName("formLayout")
        self.mnemonicLabel = QtWidgets.QLabel(AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.mnemonicLabel.setFont(font)
        self.mnemonicLabel.setObjectName("mnemonicLabel")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.LabelRole, self.mnemonicLabel)
        self.mnemonicLineEdit = QtWidgets.QLineEdit(AccountImportDialog)
        self.mnemonicLineEdit.setReadOnly(False)
        self.mnemonicLineEdit.setObjectName("mnemonicLineEdit")
        self.formLayout.setWidget(0, QtWidgets.QFormLayout.FieldRole, self.mnemonicLineEdit)
        self.mnemonicLanguageLabel = QtWidgets.QLabel(AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setItalic(False)
        font.setWeight(75)
        self.mnemonicLanguageLabel.setFont(font)
        self.mnemonicLanguageLabel.setObjectName("mnemonicLanguageLabel")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.LabelRole, self.mnemonicLanguageLabel)
        self.mnemonicLanguageComboBox = QtWidgets.QComboBox(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.mnemonicLanguageComboBox.sizePolicy().hasHeightForWidth())
        self.mnemonicLanguageComboBox.setSizePolicy(sizePolicy)
        self.mnemonicLanguageComboBox.setObjectName("mnemonicLanguageComboBox")
        self.formLayout.setWidget(1, QtWidgets.QFormLayout.FieldRole, self.mnemonicLanguageComboBox)
        self.addressLabel = QtWidgets.QLabel(AccountImportDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.addressLabel.setFont(font)
        self.addressLabel.setObjectName("addressLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.LabelRole, self.addressLabel)
        self.addressValueLabel = QtWidgets.QLabel(AccountImportDialog)
        self.addressValueLabel.setText("")
        self.addressValueLabel.setObjectName("addressValueLabel")
        self.formLayout.setWidget(2, QtWidgets.QFormLayout.FieldRole, self.addressValueLabel)
        self.verticalLayout.addLayout(self.formLayout)
        self.errorLabel = QtWidgets.QLabel(AccountImportDialog)
        self.errorLabel.setStyleSheet("color: red")
        self.errorLabel.setText("")
        self.errorLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.errorLabel.setObjectName("errorLabel")
        self.verticalLayout.addWidget(self.errorLabel)
        self.horizontalLayout = QtWidgets.QHBoxLayout()
        self.horizontalLayout.setObjectName("horizontalLayout")
        spacerItem = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem)
        self.passwordLabel = QtWidgets.QLabel(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLabel.sizePolicy().hasHeightForWidth())
        self.passwordLabel.setSizePolicy(sizePolicy)
        self.passwordLabel.setSizeIncrement(QtCore.QSize(0, 0))
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.passwordLabel.setFont(font)
        self.passwordLabel.setAlignment(QtCore.Qt.AlignLeading|QtCore.Qt.AlignLeft|QtCore.Qt.AlignVCenter)
        self.passwordLabel.setObjectName("passwordLabel")
        self.horizontalLayout.addWidget(self.passwordLabel)
        self.passwordLineEdit = QtWidgets.QLineEdit(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordLineEdit.sizePolicy().hasHeightForWidth())
        self.passwordLineEdit.setSizePolicy(sizePolicy)
        self.passwordLineEdit.setMaxLength(20)
        self.passwordLineEdit.setAlignment(QtCore.Qt.AlignCenter)
        self.passwordLineEdit.setReadOnly(True)
        self.passwordLineEdit.setObjectName("passwordLineEdit")
        self.horizontalLayout.addWidget(self.passwordLineEdit)
        self.passwordChangeButton = QtWidgets.QPushButton(AccountImportDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.passwordChangeButton.sizePolicy().hasHeightForWidth())
        self.passwordChangeButton.setSizePolicy(sizePolicy)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.passwordChangeButton.setIcon(icon)
        self.passwordChangeButton.setObjectName("passwordChangeButton")
        self.horizontalLayout.addWidget(self.passwordChangeButton)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout.addItem(spacerItem1)
        self.verticalLayout.addLayout(self.horizontalLayout)
        self.buttonBox = QtWidgets.QDialogButtonBox(AccountImportDialog)
        self.buttonBox.setOrientation(QtCore.Qt.Horizontal)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Cancel|QtWidgets.QDialogButtonBox.Ok)
        self.buttonBox.setObjectName("buttonBox")
        self.verticalLayout.addWidget(self.buttonBox, 0, QtCore.Qt.AlignHCenter)

        self.retranslateUi(AccountImportDialog)
        self.buttonBox.accepted.connect(AccountImportDialog.accept) # type: ignore
        self.buttonBox.rejected.connect(AccountImportDialog.reject) # type: ignore
        QtCore.QMetaObject.connectSlotsByName(AccountImportDialog)

    def retranslateUi(self, AccountImportDialog):
        #  _translate = QtCore.QCoreApplication.translate
        AccountImportDialog.setWindowTitle(_("Import account"))
        self.mnemonicLabel.setText(_("Mnemonic"))
        self.mnemonicLanguageLabel.setText(_("Mnemonic language"))
        self.addressLabel.setText(_("Address"))
        self.passwordLabel.setText(_("Password"))
        self.passwordChangeButton.setText(_("Change"))
import tikka.slots.pyqt.resources.icons.index_rc
