# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/windows/transfer.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_TransferDialog(object):
    def setupUi(self, TransferDialog):
        TransferDialog.setObjectName("TransferDialog")
        TransferDialog.resize(596, 167)
        self.gridLayout = QtWidgets.QGridLayout(TransferDialog)
        self.gridLayout.setObjectName("gridLayout")
        self.sendButton = QtWidgets.QPushButton(TransferDialog)
        self.sendButton.setObjectName("sendButton")
        self.gridLayout.addWidget(self.sendButton, 3, 3, 1, 1)
        self.feesButton = QtWidgets.QPushButton(TransferDialog)
        self.feesButton.setObjectName("feesButton")
        self.gridLayout.addWidget(self.feesButton, 2, 3, 1, 1)
        self.feesLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.feesLabel.setFont(font)
        self.feesLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.feesLabel.setObjectName("feesLabel")
        self.gridLayout.addWidget(self.feesLabel, 3, 0, 1, 1)
        self.amountUnitComboBox = QtWidgets.QComboBox(TransferDialog)
        self.amountUnitComboBox.setObjectName("amountUnitComboBox")
        self.gridLayout.addWidget(self.amountUnitComboBox, 2, 2, 1, 1)
        self.senderBalanceValueLabel = QtWidgets.QLabel(TransferDialog)
        self.senderBalanceValueLabel.setText("")
        self.senderBalanceValueLabel.setObjectName("senderBalanceValueLabel")
        self.gridLayout.addWidget(self.senderBalanceValueLabel, 0, 8, 1, 1)
        self.recipientBalanceValueLabel = QtWidgets.QLabel(TransferDialog)
        self.recipientBalanceValueLabel.setText("")
        self.recipientBalanceValueLabel.setObjectName("recipientBalanceValueLabel")
        self.gridLayout.addWidget(self.recipientBalanceValueLabel, 1, 8, 1, 1)
        self.amountLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.amountLabel.setFont(font)
        self.amountLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.amountLabel.setObjectName("amountLabel")
        self.gridLayout.addWidget(self.amountLabel, 2, 0, 1, 1)
        self.amountDoubleSpinBox = QtWidgets.QDoubleSpinBox(TransferDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Fixed, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.amountDoubleSpinBox.sizePolicy().hasHeightForWidth())
        self.amountDoubleSpinBox.setSizePolicy(sizePolicy)
        self.amountDoubleSpinBox.setObjectName("amountDoubleSpinBox")
        self.gridLayout.addWidget(self.amountDoubleSpinBox, 2, 1, 1, 1)
        self.recipientLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.recipientLabel.setFont(font)
        self.recipientLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.recipientLabel.setObjectName("recipientLabel")
        self.gridLayout.addWidget(self.recipientLabel, 1, 0, 1, 1)
        self.recipientAddressLineEdit = QtWidgets.QLineEdit(TransferDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.recipientAddressLineEdit.sizePolicy().hasHeightForWidth())
        self.recipientAddressLineEdit.setSizePolicy(sizePolicy)
        self.recipientAddressLineEdit.setObjectName("recipientAddressLineEdit")
        self.gridLayout.addWidget(self.recipientAddressLineEdit, 1, 1, 1, 4)
        self.sendStatusLabel = QtWidgets.QLabel(TransferDialog)
        self.sendStatusLabel.setStyleSheet("")
        self.sendStatusLabel.setText("")
        self.sendStatusLabel.setAlignment(QtCore.Qt.AlignCenter)
        self.sendStatusLabel.setObjectName("sendStatusLabel")
        self.gridLayout.addWidget(self.sendStatusLabel, 4, 1, 1, 4)
        self.senderLabel = QtWidgets.QLabel(TransferDialog)
        font = QtGui.QFont()
        font.setBold(True)
        font.setWeight(75)
        self.senderLabel.setFont(font)
        self.senderLabel.setAlignment(QtCore.Qt.AlignRight|QtCore.Qt.AlignTrailing|QtCore.Qt.AlignVCenter)
        self.senderLabel.setObjectName("senderLabel")
        self.gridLayout.addWidget(self.senderLabel, 0, 0, 1, 1)
        self.senderAddressValueLabel = QtWidgets.QLabel(TransferDialog)
        self.senderAddressValueLabel.setText("")
        self.senderAddressValueLabel.setObjectName("senderAddressValueLabel")
        self.gridLayout.addWidget(self.senderAddressValueLabel, 0, 1, 1, 4)
        self.buttonBox = QtWidgets.QDialogButtonBox(TransferDialog)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Preferred, QtWidgets.QSizePolicy.Preferred)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.buttonBox.sizePolicy().hasHeightForWidth())
        self.buttonBox.setSizePolicy(sizePolicy)
        self.buttonBox.setStandardButtons(QtWidgets.QDialogButtonBox.Close)
        self.buttonBox.setObjectName("buttonBox")
        self.gridLayout.addWidget(self.buttonBox, 4, 8, 1, 1, QtCore.Qt.AlignHCenter)
        self.loaderIconLabel = QtWidgets.QLabel(TransferDialog)
        self.loaderIconLabel.setEnabled(True)
        self.loaderIconLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.loaderIconLabel.setText("")
        self.loaderIconLabel.setScaledContents(True)
        self.loaderIconLabel.setObjectName("loaderIconLabel")
        self.gridLayout.addWidget(self.loaderIconLabel, 2, 5, 1, 1)
        self.feesValueLabel = QtWidgets.QLabel(TransferDialog)
        self.feesValueLabel.setText("")
        self.feesValueLabel.setObjectName("feesValueLabel")
        self.gridLayout.addWidget(self.feesValueLabel, 3, 1, 1, 1)

        self.retranslateUi(TransferDialog)
        QtCore.QMetaObject.connectSlotsByName(TransferDialog)

    def retranslateUi(self, TransferDialog):
        #  _translate = QtCore.QCoreApplication.translate
        TransferDialog.setWindowTitle(_("Transfer"))
        self.sendButton.setText(_("Send"))
        self.feesButton.setText(_("Fees"))
        self.feesLabel.setText(_("Fees"))
        self.amountLabel.setText(_("Amount"))
        self.recipientLabel.setText(_("Recipient"))
        self.senderLabel.setText(_("Sender"))
import tikka.slots.pyqt.resources.icons.index_rc
