# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/licence.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_LicenceWidget(object):
    def setupUi(self, LicenceWidget):
        LicenceWidget.setObjectName("LicenceWidget")
        LicenceWidget.resize(410, 303)
        self.verticalLayout = QtWidgets.QVBoxLayout(LicenceWidget)
        self.verticalLayout.setObjectName("verticalLayout")
        self.textBrowser = QtWidgets.QTextBrowser(LicenceWidget)
        self.textBrowser.setObjectName("textBrowser")
        self.verticalLayout.addWidget(self.textBrowser)

        self.retranslateUi(LicenceWidget)
        QtCore.QMetaObject.connectSlotsByName(LicenceWidget)

    def retranslateUi(self, LicenceWidget):
        #  _translate = QtCore.QCoreApplication.translate
        LicenceWidget.setWindowTitle(_("Ğ1 licence"))
