# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'tikka/slots/pyqt/resources/gui/widgets/nodes.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_NodesWidget(object):
    def setupUi(self, NodesWidget):
        NodesWidget.setObjectName("NodesWidget")
        NodesWidget.resize(724, 526)
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(NodesWidget)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        self.gridLayout = QtWidgets.QGridLayout()
        self.gridLayout.setObjectName("gridLayout")
        self.label = QtWidgets.QLabel(NodesWidget)
        self.label.setObjectName("label")
        self.gridLayout.addWidget(self.label, 0, 0, 1, 1)
        self.entryPointComboBox = QtWidgets.QComboBox(NodesWidget)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.entryPointComboBox.sizePolicy().hasHeightForWidth())
        self.entryPointComboBox.setSizePolicy(sizePolicy)
        self.entryPointComboBox.setToolTip("")
        self.entryPointComboBox.setEditable(True)
        self.entryPointComboBox.setMinimumContentsLength(0)
        self.entryPointComboBox.setObjectName("entryPointComboBox")
        self.gridLayout.addWidget(self.entryPointComboBox, 0, 1, 1, 1)
        self.connectButton = QtWidgets.QPushButton(NodesWidget)
        self.connectButton.setObjectName("connectButton")
        self.gridLayout.addWidget(self.connectButton, 0, 2, 1, 1)
        self.connectionStatusLabel = QtWidgets.QLabel(NodesWidget)
        self.connectionStatusLabel.setMaximumSize(QtCore.QSize(16, 16))
        self.connectionStatusLabel.setText("")
        self.connectionStatusLabel.setPixmap(QtGui.QPixmap(":/icons/disconnected"))
        self.connectionStatusLabel.setScaledContents(True)
        self.connectionStatusLabel.setObjectName("connectionStatusLabel")
        self.gridLayout.addWidget(self.connectionStatusLabel, 0, 3, 1, 1)
        self.refreshNodesButton = QtWidgets.QPushButton(NodesWidget)
        icon = QtGui.QIcon()
        icon.addPixmap(QtGui.QPixmap(":/icons/refresh_icon"), QtGui.QIcon.Normal, QtGui.QIcon.Off)
        self.refreshNodesButton.setIcon(icon)
        self.refreshNodesButton.setObjectName("refreshNodesButton")
        self.gridLayout.addWidget(self.refreshNodesButton, 0, 4, 1, 1)
        self.verticalLayout_2.addLayout(self.gridLayout)
        self.nodesTableView = QtWidgets.QTableView(NodesWidget)
        self.nodesTableView.setContextMenuPolicy(QtCore.Qt.CustomContextMenu)
        self.nodesTableView.setEditTriggers(QtWidgets.QAbstractItemView.NoEditTriggers)
        self.nodesTableView.setSortingEnabled(True)
        self.nodesTableView.setObjectName("nodesTableView")
        self.nodesTableView.horizontalHeader().setStretchLastSection(True)
        self.nodesTableView.verticalHeader().setVisible(False)
        self.verticalLayout_2.addWidget(self.nodesTableView)

        self.retranslateUi(NodesWidget)
        QtCore.QMetaObject.connectSlotsByName(NodesWidget)

    def retranslateUi(self, NodesWidget):
        #  _translate = QtCore.QCoreApplication.translate
        NodesWidget.setWindowTitle(_("Network nodes"))
        self.label.setText(_("Entry point"))
        self.connectButton.setToolTip(_("Connection to entry point"))
        self.connectButton.setText(_("Connect"))
        self.refreshNodesButton.setToolTip(_("Refresh nodes from network"))
        self.refreshNodesButton.setText(_("Refresh"))
import tikka.slots.pyqt.resources.icons.index_rc
